@@ -24,7 +24,7 @@ use self::ISO2022JPState::{ASCII,Katakana,Lead};
  * the upper half of JIS X 0212 in G2 (`8E [A1-DF]`), and
  * JIS X 0212 in G3 (`8F [A1-FE] [A1-FE]`).
  */
-#[deriving(Clone)]
+#[deriving(Clone, Copy)]
 pub struct EUCJPEncoding;
 
 impl Encoding for EUCJPEncoding {
@@ -35,7 +35,7 @@ impl Encoding for EUCJPEncoding {
 }
 
 /// An encoder for EUC-JP with unused G3 character set.
-#[deriving(Clone)]
+#[deriving(Clone, Copy)]
 pub struct EUCJPEncoder;
 
 impl EUCJPEncoder {
@@ -51,10 +51,10 @@ impl RawEncoder for EUCJPEncoder {
 
         for ((i,j), ch) in input.index_iter() {
             match ch {
-                '\u0000'...'\u007f' => { output.write_byte(ch as u8); }
-                '\u00a5' => { output.write_byte(0x5c); }
-                '\u203e' => { output.write_byte(0x7e); }
-                '\uff61'...'\uff9f' => {
+                '\u{0}'...'\u{7f}' => { output.write_byte(ch as u8); }
+                '\u{a5}' => { output.write_byte(0x5c); }
+                '\u{203e}' => { output.write_byte(0x7e); }
+                '\u{ff61}'...'\u{ff9f}' => {
                     output.write_byte(0x8e);
                     output.write_byte((ch as uint - 0xff61 + 0xa1) as u8);
                 }
@@ -83,7 +83,7 @@ impl RawEncoder for EUCJPEncoder {
 
 ascii_compatible_stateful_decoder! {
     #[doc="A decoder for EUC-JP with JIS X 0212 in G3."]
-    #[deriving(Clone)]
+    #[deriving(Clone, Copy)]
     struct EUCJP0212Decoder;
 
     module eucjp;
@@ -165,8 +165,6 @@ mod eucjp_tests {
     use std::iter::range_inclusive;
     use testutils;
     use types::*;
-    use types::DecoderTrap::Strict as DecodeStrict;
-    use types::EncoderTrap::Strict as EncodeStrict;
 
     #[test]
     fn test_encoder_valid() {
@@ -174,11 +172,11 @@ mod eucjp_tests {
         assert_feed_ok!(e, "A", "", [0x41]);
         assert_feed_ok!(e, "BC", "", [0x42, 0x43]);
         assert_feed_ok!(e, "", "", []);
-        assert_feed_ok!(e, "\u00a5", "", [0x5c]);
-        assert_feed_ok!(e, "\u203e", "", [0x7e]);
-        assert_feed_ok!(e, "\u306b\u307b\u3093", "", [0xa4, 0xcb, 0xa4, 0xdb, 0xa4, 0xf3]);
-        assert_feed_ok!(e, "\uff86\uff8e\uff9d", "", [0x8e, 0xc6, 0x8e, 0xce, 0x8e, 0xdd]);
-        assert_feed_ok!(e, "\u65e5\u672c", "", [0xc6, 0xfc, 0xcb, 0xdc]);
+        assert_feed_ok!(e, "\u{a5}", "", [0x5c]);
+        assert_feed_ok!(e, "\u{203e}", "", [0x7e]);
+        assert_feed_ok!(e, "\u{306b}\u{307b}\u{3093}", "", [0xa4, 0xcb, 0xa4, 0xdb, 0xa4, 0xf3]);
+        assert_feed_ok!(e, "\u{ff86}\u{ff8e}\u{ff9d}", "", [0x8e, 0xc6, 0x8e, 0xce, 0x8e, 0xdd]);
+        assert_feed_ok!(e, "\u{65e5}\u{672c}", "", [0xc6, 0xfc, 0xcb, 0xdc]);
         assert_finish_ok!(e, []);
     }
 
@@ -187,17 +185,17 @@ mod eucjp_tests {
         // these characters are double-mapped to both EUDC area and Shift_JIS extension area
         // but only the former should be used. (note that U+FFE2 is triple-mapped!)
         let mut e = EUCJPEncoding.raw_encoder();
-        assert_feed_ok!(e, "\u9ed1\u2170\uffe2", "", [0xfc, 0xee, 0xfc, 0xf1, 0xa2, 0xcc]);
+        assert_feed_ok!(e, "\u{9ed1}\u{2170}\u{ffe2}", "", [0xfc, 0xee, 0xfc, 0xf1, 0xa2, 0xcc]);
         assert_finish_ok!(e, []);
     }
 
     #[test]
     fn test_encoder_invalid() {
         let mut e = EUCJPEncoding.raw_encoder();
-        assert_feed_err!(e, "", "\uffff", "", []);
-        assert_feed_err!(e, "?", "\uffff", "!", [0x3f]);
+        assert_feed_err!(e, "", "\u{ffff}", "", []);
+        assert_feed_err!(e, "?", "\u{ffff}", "!", [0x3f]);
         // JIS X 0212 is not supported in the encoder
-        assert_feed_err!(e, "", "\u736c", "\u8c78", []);
+        assert_feed_err!(e, "", "\u{736c}", "\u{8c78}", []);
         assert_finish_ok!(e, []);
     }
 
@@ -209,10 +207,10 @@ mod eucjp_tests {
         assert_feed_ok!(d, [], [], "");
         assert_feed_ok!(d, [0x5c], [], "\\");
         assert_feed_ok!(d, [0x7e], [], "~");
-        assert_feed_ok!(d, [0xa4, 0xcb, 0xa4, 0xdb, 0xa4, 0xf3], [], "\u306b\u307b\u3093");
-        assert_feed_ok!(d, [0x8e, 0xc6, 0x8e, 0xce, 0x8e, 0xdd], [], "\uff86\uff8e\uff9d");
-        assert_feed_ok!(d, [0xc6, 0xfc, 0xcb, 0xdc], [], "\u65e5\u672c");
-        assert_feed_ok!(d, [0x8f, 0xcb, 0xc6, 0xec, 0xb8], [], "\u736c\u8c78");
+        assert_feed_ok!(d, [0xa4, 0xcb, 0xa4, 0xdb, 0xa4, 0xf3], [], "\u{306b}\u{307b}\u{3093}");
+        assert_feed_ok!(d, [0x8e, 0xc6, 0x8e, 0xce, 0x8e, 0xdd], [], "\u{ff86}\u{ff8e}\u{ff9d}");
+        assert_feed_ok!(d, [0xc6, 0xfc, 0xcb, 0xdc], [], "\u{65e5}\u{672c}");
+        assert_feed_ok!(d, [0x8f, 0xcb, 0xc6, 0xec, 0xb8], [], "\u{736c}\u{8c78}");
         assert_finish_ok!(d, "");
     }
 
@@ -220,22 +218,22 @@ mod eucjp_tests {
     fn test_decoder_valid_partial() {
         let mut d = EUCJPEncoding.raw_decoder();
         assert_feed_ok!(d, [], [0xa4], "");
-        assert_feed_ok!(d, [0xcb], [0xa4], "\u306b");
-        assert_feed_ok!(d, [0xdb], [0xa4], "\u307b");
-        assert_feed_ok!(d, [0xf3], [], "\u3093");
+        assert_feed_ok!(d, [0xcb], [0xa4], "\u{306b}");
+        assert_feed_ok!(d, [0xdb], [0xa4], "\u{307b}");
+        assert_feed_ok!(d, [0xf3], [], "\u{3093}");
         assert_feed_ok!(d, [], [0x8e], "");
-        assert_feed_ok!(d, [0xc6], [0x8e], "\uff86");
-        assert_feed_ok!(d, [0xce], [0x8e], "\uff8e");
-        assert_feed_ok!(d, [0xdd], [], "\uff9d");
+        assert_feed_ok!(d, [0xc6], [0x8e], "\u{ff86}");
+        assert_feed_ok!(d, [0xce], [0x8e], "\u{ff8e}");
+        assert_feed_ok!(d, [0xdd], [], "\u{ff9d}");
         assert_feed_ok!(d, [], [0xc6], "");
-        assert_feed_ok!(d, [0xfc], [0xcb], "\u65e5");
-        assert_feed_ok!(d, [0xdc], [], "\u672c");
+        assert_feed_ok!(d, [0xfc], [0xcb], "\u{65e5}");
+        assert_feed_ok!(d, [0xdc], [], "\u{672c}");
         assert_feed_ok!(d, [], [0x8f], "");
         assert_feed_ok!(d, [], [0xcb], "");
-        assert_feed_ok!(d, [0xc6], [0xec], "\u736c");
-        assert_feed_ok!(d, [0xb8], [], "\u8c78");
+        assert_feed_ok!(d, [0xc6], [0xec], "\u{736c}");
+        assert_feed_ok!(d, [0xb8], [], "\u{8c78}");
         assert_feed_ok!(d, [], [0x8f, 0xcb], "");
-        assert_feed_ok!(d, [0xc6, 0xec, 0xb8], [], "\u736c\u8c78");
+        assert_feed_ok!(d, [0xc6, 0xec, 0xb8], [], "\u{736c}\u{8c78}");
         assert_finish_ok!(d, "");
     }
 
@@ -377,9 +375,9 @@ mod eucjp_tests {
     #[test]
     fn test_decoder_feed_after_finish() {
         let mut d = EUCJPEncoding.raw_decoder();
-        assert_feed_ok!(d, [0xa4, 0xa2], [0xa4], "\u3042");
+        assert_feed_ok!(d, [0xa4, 0xa2], [0xa4], "\u{3042}");
         assert_finish_err!(d, "");
-        assert_feed_ok!(d, [0xa4, 0xa2], [], "\u3042");
+        assert_feed_ok!(d, [0xa4, 0xa2], [], "\u{3042}");
         assert_finish_ok!(d, "");
     }
 
@@ -388,16 +386,17 @@ mod eucjp_tests {
         let s = testutils::JAPANESE_TEXT;
         bencher.bytes = s.len() as u64;
         bencher.iter(|| test::black_box({
-            EUCJPEncoding.encode(s[], EncodeStrict)
+            EUCJPEncoding.encode(s[], EncoderTrap::Strict)
         }))
     }
 
     #[bench]
     fn bench_decode_short_text(bencher: &mut test::Bencher) {
-        let s = EUCJPEncoding.encode(testutils::JAPANESE_TEXT, EncodeStrict).ok().unwrap();
+        let s = EUCJPEncoding.encode(testutils::JAPANESE_TEXT,
+                                     EncoderTrap::Strict).ok().unwrap();
         bencher.bytes = s.len() as u64;
         bencher.iter(|| test::black_box({
-            EUCJPEncoding.decode(s[], DecodeStrict)
+            EUCJPEncoding.decode(s[], DecoderTrap::Strict)
         }))
     }
 }
@@ -417,7 +416,7 @@ mod eucjp_tests {
  * It requires some cares to handle
  * since the second byte of JIS X 0208 can have its MSB unset.
  */
-#[deriving(Clone)]
+#[deriving(Clone, Copy)]
 pub struct Windows31JEncoding;
 
 impl Encoding for Windows31JEncoding {
@@ -428,7 +427,7 @@ impl Encoding for Windows31JEncoding {
 }
 
 /// An encoder for Shift_JIS with IBM/NEC extensions.
-#[deriving(Clone)]
+#[deriving(Clone, Copy)]
 pub struct Windows31JEncoder;
 
 impl Windows31JEncoder {
@@ -444,10 +443,10 @@ impl RawEncoder for Windows31JEncoder {
 
         for ((i,j), ch) in input.index_iter() {
             match ch {
-                '\u0000'...'\u0080' => { output.write_byte(ch as u8); }
-                '\u00a5' => { output.write_byte(0x5c); }
-                '\u203e' => { output.write_byte(0x7e); }
-                '\uff61'...'\uff9f' => { output.write_byte((ch as uint - 0xff61 + 0xa1) as u8); }
+                '\u{0}'...'\u{80}' => { output.write_byte(ch as u8); }
+                '\u{a5}' => { output.write_byte(0x5c); }
+                '\u{203e}' => { output.write_byte(0x7e); }
+                '\u{ff61}'...'\u{ff9f}' => { output.write_byte((ch as uint - 0xff61 + 0xa1) as u8); }
                 _ => {
                     // corresponds to the "index shift_jis pointer" in the WHATWG spec
                     let ptr = index::jis0208::backward_remapped(ch as u32);
@@ -476,7 +475,7 @@ impl RawEncoder for Windows31JEncoder {
 
 ascii_compatible_stateful_decoder! {
     #[doc="A decoder for Shift_JIS with IBM/NEC extensions."]
-    #[deriving(Clone)]
+    #[deriving(Clone, Copy)]
     struct Windows31JDecoder;
 
     module windows31j;
@@ -523,8 +522,6 @@ mod windows31j_tests {
     use std::iter::range_inclusive;
     use testutils;
     use types::*;
-    use types::DecoderTrap::Strict as DecodeStrict;
-    use types::EncoderTrap::Strict as EncodeStrict;
 
     #[test]
     fn test_encoder_valid() {
@@ -532,20 +529,20 @@ mod windows31j_tests {
         assert_feed_ok!(e, "A", "", [0x41]);
         assert_feed_ok!(e, "BC", "", [0x42, 0x43]);
         assert_feed_ok!(e, "", "", []);
-        assert_feed_ok!(e, "\u00a5", "", [0x5c]);
-        assert_feed_ok!(e, "\u203e", "", [0x7e]);
-        assert_feed_ok!(e, "\u306b\u307b\u3093", "", [0x82, 0xc9, 0x82, 0xd9, 0x82, 0xf1]);
-        assert_feed_ok!(e, "\uff86\uff8e\uff9d", "", [0xc6, 0xce, 0xdd]);
-        assert_feed_ok!(e, "\u65e5\u672c", "", [0x93, 0xfa, 0x96, 0x7b]);
+        assert_feed_ok!(e, "\u{a5}", "", [0x5c]);
+        assert_feed_ok!(e, "\u{203e}", "", [0x7e]);
+        assert_feed_ok!(e, "\u{306b}\u{307b}\u{3093}", "", [0x82, 0xc9, 0x82, 0xd9, 0x82, 0xf1]);
+        assert_feed_ok!(e, "\u{ff86}\u{ff8e}\u{ff9d}", "", [0xc6, 0xce, 0xdd]);
+        assert_feed_ok!(e, "\u{65e5}\u{672c}", "", [0x93, 0xfa, 0x96, 0x7b]);
         assert_finish_ok!(e, []);
     }
 
     #[test]
     fn test_encoder_no_eudc() {
         let mut e = Windows31JEncoding.raw_encoder();
-        assert_feed_err!(e, "", "\ue000", "", []);
-        assert_feed_err!(e, "", "\ue757", "", []);
-        assert_feed_err!(e, "", "\ue758", "", []);
+        assert_feed_err!(e, "", "\u{e000}", "", []);
+        assert_feed_err!(e, "", "\u{e757}", "", []);
+        assert_feed_err!(e, "", "\u{e758}", "", []);
         assert_finish_ok!(e, []);
     }
 
@@ -554,16 +551,16 @@ mod windows31j_tests {
         // these characters are double-mapped to both EUDC area and Shift_JIS extension area
         // but only the latter should be used. (note that U+FFE2 is triple-mapped!)
         let mut e = Windows31JEncoding.raw_encoder();
-        assert_feed_ok!(e, "\u9ed1\u2170\uffe2", "", [0xfc, 0x4b, 0xfa, 0x40, 0x81, 0xca]);
+        assert_feed_ok!(e, "\u{9ed1}\u{2170}\u{ffe2}", "", [0xfc, 0x4b, 0xfa, 0x40, 0x81, 0xca]);
         assert_finish_ok!(e, []);
     }
 
     #[test]
     fn test_encoder_invalid() {
         let mut e = Windows31JEncoding.raw_encoder();
-        assert_feed_err!(e, "", "\uffff", "", []);
-        assert_feed_err!(e, "?", "\uffff", "!", [0x3f]);
-        assert_feed_err!(e, "", "\u736c", "\u8c78", []);
+        assert_feed_err!(e, "", "\u{ffff}", "", []);
+        assert_feed_err!(e, "?", "\u{ffff}", "!", [0x3f]);
+        assert_feed_err!(e, "", "\u{736c}", "\u{8c78}", []);
         assert_finish_ok!(e, []);
     }
 
@@ -575,10 +572,10 @@ mod windows31j_tests {
         assert_feed_ok!(d, [], [], "");
         assert_feed_ok!(d, [0x5c], [], "\\");
         assert_feed_ok!(d, [0x7e], [], "~");
-        assert_feed_ok!(d, [0x80], [], "\u0080"); // compatibility
-        assert_feed_ok!(d, [0x82, 0xc9, 0x82, 0xd9, 0x82, 0xf1], [], "\u306b\u307b\u3093");
-        assert_feed_ok!(d, [0xc6, 0xce, 0xdd], [], "\uff86\uff8e\uff9d");
-        assert_feed_ok!(d, [0x93, 0xfa, 0x96, 0x7b], [], "\u65e5\u672c");
+        assert_feed_ok!(d, [0x80], [], "\u{80}"); // compatibility
+        assert_feed_ok!(d, [0x82, 0xc9, 0x82, 0xd9, 0x82, 0xf1], [], "\u{306b}\u{307b}\u{3093}");
+        assert_feed_ok!(d, [0xc6, 0xce, 0xdd], [], "\u{ff86}\u{ff8e}\u{ff9d}");
+        assert_feed_ok!(d, [0x93, 0xfa, 0x96, 0x7b], [], "\u{65e5}\u{672c}");
         assert_finish_ok!(d, "");
     }
 
@@ -586,8 +583,8 @@ mod windows31j_tests {
     fn test_decoder_eudc() {
         let mut d = Windows31JEncoding.raw_decoder();
         assert_feed_ok!(d, [], [0xf0], "");
-        assert_feed_ok!(d, [0x40], [], "\ue000");
-        assert_feed_ok!(d, [0xf9, 0xfc], [], "\ue757");
+        assert_feed_ok!(d, [0x40], [], "\u{e000}");
+        assert_feed_ok!(d, [0xf9, 0xfc], [], "\u{e757}");
         assert_feed_err!(d, [], [0xf0], [0x00], "");
         assert_feed_err!(d, [], [0xf0], [0xff], "");
         assert_finish_ok!(d, "");
@@ -674,9 +671,9 @@ mod windows31j_tests {
     #[test]
     fn test_decoder_feed_after_finish() {
         let mut d = Windows31JEncoding.raw_decoder();
-        assert_feed_ok!(d, [0x82, 0xa0], [0x82], "\u3042");
+        assert_feed_ok!(d, [0x82, 0xa0], [0x82], "\u{3042}");
         assert_finish_err!(d, "");
-        assert_feed_ok!(d, [0x82, 0xa0], [], "\u3042");
+        assert_feed_ok!(d, [0x82, 0xa0], [], "\u{3042}");
         assert_finish_ok!(d, "");
     }
 
@@ -685,16 +682,17 @@ mod windows31j_tests {
         let s = testutils::JAPANESE_TEXT;
         bencher.bytes = s.len() as u64;
         bencher.iter(|| test::black_box({
-            Windows31JEncoding.encode(s[], EncodeStrict)
+            Windows31JEncoding.encode(s[], EncoderTrap::Strict)
         }))
     }
 
     #[bench]
     fn bench_decode_short_text(bencher: &mut test::Bencher) {
-        let s = Windows31JEncoding.encode(testutils::JAPANESE_TEXT, EncodeStrict).ok().unwrap();
+        let s = Windows31JEncoding.encode(testutils::JAPANESE_TEXT,
+                                          EncoderTrap::Strict).ok().unwrap();
         bencher.bytes = s.len() as u64;
         bencher.iter(|| test::black_box({
-            Windows31JEncoding.decode(s[], DecodeStrict)
+            Windows31JEncoding.decode(s[], DecoderTrap::Strict)
         }))
     }
 }
@@ -712,7 +710,7 @@ mod windows31j_tests {
  *   but willfully violated)
  * - JIS X 0212-1990 (`ESC $ ( D`, XXX asymmetric support)
  */
-#[deriving(Clone)]
+#[deriving(Clone, Copy)]
 pub struct ISO2022JPEncoding;
 
 impl Encoding for ISO2022JPEncoding {
@@ -722,7 +720,7 @@ impl Encoding for ISO2022JPEncoding {
     fn raw_decoder(&self) -> Box<RawDecoder> { ISO2022JPDecoder::new() }
 }
 
-#[deriving(PartialEq,Clone)]
+#[deriving(PartialEq,Clone,Copy)]
 enum ISO2022JPState {
     ASCII, // U+0000..007F, U+00A5, U+203E
     Katakana, // JIS X 0201: U+FF61..FF9F
@@ -730,7 +728,7 @@ enum ISO2022JPState {
 }
 
 /// An encoder for ISO-2022-JP without JIS X 0212/0213 support.
-#[deriving(Clone)]
+#[deriving(Clone, Copy)]
 pub struct ISO2022JPEncoder {
     st: ISO2022JPState
 }
@@ -759,10 +757,10 @@ impl RawEncoder for ISO2022JPEncoder {
 
         for ((i,j), ch) in input.index_iter() {
             match ch {
-                '\u0000'...'\u007f' => { ensure_ASCII!(); output.write_byte(ch as u8); }
-                '\u00a5' => { ensure_ASCII!(); output.write_byte(0x5c); }
-                '\u203e' => { ensure_ASCII!(); output.write_byte(0x7e); }
-                '\uff61'...'\uff9f' => {
+                '\u{0}'...'\u{7f}' => { ensure_ASCII!(); output.write_byte(ch as u8); }
+                '\u{a5}' => { ensure_ASCII!(); output.write_byte(0x5c); }
+                '\u{203e}' => { ensure_ASCII!(); output.write_byte(0x7e); }
+                '\u{ff61}'...'\u{ff9f}' => {
                     ensure_Katakana!();
                     output.write_byte((ch as uint - 0xff61 + 0x21) as u8);
                 }
@@ -795,7 +793,7 @@ impl RawEncoder for ISO2022JPEncoder {
 
 stateful_decoder! {
     #[doc="A decoder for ISO-2022-JP with JIS X 0212 support."]
-    #[deriving(Clone)]
+    #[deriving(Clone, Copy)]
     struct ISO2022JPDecoder;
 
     module iso2022jp;
@@ -922,9 +920,6 @@ mod iso2022jp_tests {
     use super::ISO2022JPEncoding;
     use testutils;
     use types::*;
-    use types::DecoderTrap::Replace as DecodeReplace;
-    use types::DecoderTrap::Strict as DecodeStrict;
-    use types::EncoderTrap::Strict as EncodeStrict;
 
     #[test]
     fn test_encoder_valid() {
@@ -933,13 +928,13 @@ mod iso2022jp_tests {
         assert_feed_ok!(e, "BC", "", [0x42, 0x43]);
         assert_feed_ok!(e, "\x1b\x24\x42", "", [0x1b, 0x24, 0x42]); // no round-trip guarantee
         assert_feed_ok!(e, "", "", []);
-        assert_feed_ok!(e, "\u00a5", "", [0x5c]);
-        assert_feed_ok!(e, "\u203e", "", [0x7e]);
-        assert_feed_ok!(e, "\u306b\u307b\u3093", "", [0x1b, 0x24, 0x42,
-                                                      0x24, 0x4b, 0x24, 0x5b, 0x24, 0x73]);
-        assert_feed_ok!(e, "\u65e5\u672c", "", [0x46, 0x7c, 0x4b, 0x5c]);
-        assert_feed_ok!(e, "\uff86\uff8e\uff9d", "", [0x1b, 0x28, 0x49,
-                                                      0x46, 0x4e, 0x5d]);
+        assert_feed_ok!(e, "\u{a5}", "", [0x5c]);
+        assert_feed_ok!(e, "\u{203e}", "", [0x7e]);
+        assert_feed_ok!(e, "\u{306b}\u{307b}\u{3093}", "", [0x1b, 0x24, 0x42,
+                                                            0x24, 0x4b, 0x24, 0x5b, 0x24, 0x73]);
+        assert_feed_ok!(e, "\u{65e5}\u{672c}", "", [0x46, 0x7c, 0x4b, 0x5c]);
+        assert_feed_ok!(e, "\u{ff86}\u{ff8e}\u{ff9d}", "", [0x1b, 0x28, 0x49,
+                                                            0x46, 0x4e, 0x5d]);
         assert_feed_ok!(e, "XYZ", "", [0x1b, 0x28, 0x42,
                                        0x58, 0x59, 0x5a]);
         assert_finish_ok!(e, []);
@@ -951,8 +946,8 @@ mod iso2022jp_tests {
         // - D is omitted as the encoder does not support JIS X 0212.
         // a (3,2) De Bruijn near-sequence "ABCACBA" is used to test all possible cases.
         static AD: &'static str = "\x20";
-        static BD: &'static str = "\u30cd";
-        static CD: &'static str = "\uff88";
+        static BD: &'static str = "\u{30cd}";
+        static CD: &'static str = "\u{ff88}";
         static AE: &'static [u8] = &[0x1b, 0x28, 0x42, 0x20];
         static BE: &'static [u8] = &[0x1b, 0x24, 0x42, 0x25, 0x4d];
         static CE: &'static [u8] = &[0x1b, 0x28, 0x49, 0x48];
@@ -966,10 +961,10 @@ mod iso2022jp_tests {
     #[test]
     fn test_encoder_invalid() {
         let mut e = ISO2022JPEncoding.raw_encoder();
-        assert_feed_err!(e, "", "\uffff", "", []);
-        assert_feed_err!(e, "?", "\uffff", "!", [0x3f]);
+        assert_feed_err!(e, "", "\u{ffff}", "", []);
+        assert_feed_err!(e, "?", "\u{ffff}", "!", [0x3f]);
         // JIS X 0212 is not supported in the encoder
-        assert_feed_err!(e, "", "\u736c", "\u8c78", []);
+        assert_feed_err!(e, "", "\u{736c}", "\u{8c78}", []);
         assert_finish_ok!(e, []);
     }
 
@@ -986,31 +981,31 @@ mod iso2022jp_tests {
         assert_feed_ok!(d, [0x1b, 0x24, 0x42,
                             0x24, 0x4b,
                             0x1b, 0x24, 0x42,
-                            0x24, 0x5b, 0x24, 0x73], [], "\u306b\u307b\u3093");
-        assert_feed_ok!(d, [0x46, 0x7c, 0x4b, 0x5c], [], "\u65e5\u672c");
+                            0x24, 0x5b, 0x24, 0x73], [], "\u{306b}\u{307b}\u{3093}");
+        assert_feed_ok!(d, [0x46, 0x7c, 0x4b, 0x5c], [], "\u{65e5}\u{672c}");
         assert_feed_ok!(d, [0x1b, 0x28, 0x49,
-                            0x46, 0x4e, 0x5d], [], "\uff86\uff8e\uff9d");
+                            0x46, 0x4e, 0x5d], [], "\u{ff86}\u{ff8e}\u{ff9d}");
         assert_feed_ok!(d, [0x1b, 0x24, 0x28, 0x44,
                             0x4b, 0x46,
                             0x1b, 0x24, 0x40,
-                            0x6c, 0x38], [], "\u736c\u8c78");
+                            0x6c, 0x38], [], "\u{736c}\u{8c78}");
         assert_feed_ok!(d, [0x1b, 0x28, 0x42,
                             0x58, 0x59, 0x5a], [], "XYZ");
         assert_finish_ok!(d, "");
 
         let mut d = ISO2022JPEncoding.raw_decoder();
         assert_feed_ok!(d, [0x1b, 0x24, 0x42,
-                            0x24, 0x4b, 0x24, 0x5b, 0x24, 0x73], [], "\u306b\u307b\u3093");
+                            0x24, 0x4b, 0x24, 0x5b, 0x24, 0x73], [], "\u{306b}\u{307b}\u{3093}");
         assert_finish_ok!(d, "");
 
         let mut d = ISO2022JPEncoding.raw_decoder();
         assert_feed_ok!(d, [0x1b, 0x28, 0x49,
-                            0x46, 0x4e, 0x5d], [], "\uff86\uff8e\uff9d");
+                            0x46, 0x4e, 0x5d], [], "\u{ff86}\u{ff8e}\u{ff9d}");
         assert_finish_ok!(d, "");
 
         let mut d = ISO2022JPEncoding.raw_decoder();
         assert_feed_ok!(d, [0x1b, 0x24, 0x28, 0x44,
-                            0x4b, 0x46], [], "\u736c");
+                            0x4b, 0x46], [], "\u{736c}");
         assert_finish_ok!(d, "");
 
         // one ASCII character and three similarly looking characters:
@@ -1020,9 +1015,9 @@ mod iso2022jp_tests {
         // - D: U+793B CJK UNIFIED IDEOGRAPH-793B (requires JIS X 0212 Lead state)
         // a (4,2) De Bruijn sequence "AABBCCACBADDBDCDA" is used to test all possible cases.
         static AD: &'static str = "\x20";
-        static BD: &'static str = "\u30cd";
-        static CD: &'static str = "\uff88";
-        static DD: &'static str = "\u793b";
+        static BD: &'static str = "\u{30cd}";
+        static CD: &'static str = "\u{ff88}";
+        static DD: &'static str = "\u{793b}";
         static AE: &'static [u8] = &[0x1b, 0x28, 0x42,       0x20];
         static BE: &'static [u8] = &[0x1b, 0x24, 0x42,       0x25, 0x4d];
         static CE: &'static [u8] = &[0x1b, 0x28, 0x49,       0x48];
@@ -1048,23 +1043,23 @@ mod iso2022jp_tests {
         assert_feed_ok!(d, [], [0x1b], "");
         assert_feed_ok!(d, [], [0x24], "");
         assert_feed_ok!(d, [0x42], [0x24], "");
-        assert_feed_ok!(d, [0x4b], [0x1b, 0x24], "\u306b");
-        assert_feed_ok!(d, [0x42, 0x24, 0x5b], [], "\u307b");
+        assert_feed_ok!(d, [0x4b], [0x1b, 0x24], "\u{306b}");
+        assert_feed_ok!(d, [0x42, 0x24, 0x5b], [], "\u{307b}");
         assert_feed_ok!(d, [], [0x1b], "");
-        assert_feed_ok!(d, [0x24, 0x42, 0x24, 0x73], [], "\u3093");
+        assert_feed_ok!(d, [0x24, 0x42, 0x24, 0x73], [], "\u{3093}");
 
         assert_feed_ok!(d, [], [0x1b], "");
         assert_feed_ok!(d, [], [0x28], "");
-        assert_feed_ok!(d, [0x49, 0x46], [], "\uff86");
+        assert_feed_ok!(d, [0x49, 0x46], [], "\u{ff86}");
         assert_feed_ok!(d, [], [0x1b, 0x28], "");
-        assert_feed_ok!(d, [0x49, 0x4e], [0x1b], "\uff8e");
-        assert_feed_ok!(d, [0x28, 0x49, 0x5d], [], "\uff9d");
+        assert_feed_ok!(d, [0x49, 0x4e], [0x1b], "\u{ff8e}");
+        assert_feed_ok!(d, [0x28, 0x49, 0x5d], [], "\u{ff9d}");
 
         assert_feed_ok!(d, [], [0x1b, 0x24], "");
         assert_feed_ok!(d, [], [0x28], "");
         assert_feed_ok!(d, [0x44], [0x4b], "");
-        assert_feed_ok!(d, [0x46], [0x1b, 0x24, 0x28], "\u736c");
-        assert_feed_ok!(d, [0x44, 0x4b, 0x46], [], "\u736c");
+        assert_feed_ok!(d, [0x46], [0x1b, 0x24, 0x28], "\u{736c}");
+        assert_feed_ok!(d, [0x44, 0x4b, 0x46], [], "\u{736c}");
 
         assert_finish_ok!(d, "");
     }
@@ -1076,16 +1071,16 @@ mod iso2022jp_tests {
         assert_feed_ok!(d, [0x1b, 0x24, 0x42,
                             0x25, 0x4d,
                             0x0a,
-                            0x25, 0x4d], [], "\u30cd\n\x25\x4d");
+                            0x25, 0x4d], [], "\u{30cd}\n\x25\x4d");
         assert_feed_ok!(d, [0x1b, 0x24, 0x28, 0x44,
                             0x50, 0x4b,
                             0x0a,
-                            0x50, 0x4b], [], "\u793b\n\x50\x4b");
+                            0x50, 0x4b], [], "\u{793b}\n\x50\x4b");
         assert_finish_ok!(d, "");
 
         // other states don't allow CR
         let mut d = ISO2022JPEncoding.raw_decoder();
-        assert_feed_err!(d, [0x1b, 0x28, 0x49, 0x48], [0x0a], [], "\uff88"); // Katakana
+        assert_feed_err!(d, [0x1b, 0x28, 0x49, 0x48], [0x0a], [], "\u{ff88}"); // Katakana
         assert_feed_err!(d, [0x1b, 0x24, 0x42], [0x25, 0x0a], [], ""); // Trail
         assert_finish_ok!(d, "");
     }
@@ -1093,11 +1088,11 @@ mod iso2022jp_tests {
     #[test]
     fn test_decoder_invalid_partial() {
         let mut d = ISO2022JPEncoding.raw_decoder();
-        assert_feed_ok!(d, [0x1b, 0x24, 0x42, 0x24, 0x4b], [0x24], "\u306b");
+        assert_feed_ok!(d, [0x1b, 0x24, 0x42, 0x24, 0x4b], [0x24], "\u{306b}");
         assert_finish_err!(d, "");
 
         let mut d = ISO2022JPEncoding.raw_decoder();
-        assert_feed_ok!(d, [0x1b, 0x24, 0x28, 0x44, 0x4b, 0x46], [0x50], "\u736c");
+        assert_feed_ok!(d, [0x1b, 0x24, 0x28, 0x44, 0x4b, 0x46], [0x50], "\u{736c}");
         assert_finish_err!(d, "");
     }
 
@@ -1119,14 +1114,14 @@ mod iso2022jp_tests {
         assert_feed_ok!(d, [], [0x1b, 0x28], "");
         assert_finish_err!(d, ""); // no backup
 
-        assert_eq!(ISO2022JPEncoding.decode([0x1b][], DecodeReplace),
-                   Ok("\ufffd".to_string()));
-        assert_eq!(ISO2022JPEncoding.decode([0x1b, 0x24][], DecodeReplace),
-                   Ok("\ufffd".to_string()));
-        assert_eq!(ISO2022JPEncoding.decode([0x1b, 0x24, 0x28][], DecodeReplace),
-                   Ok("\ufffd\x28".to_string()));
-        assert_eq!(ISO2022JPEncoding.decode([0x1b, 0x28][], DecodeReplace),
-                   Ok("\ufffd".to_string()));
+        assert_eq!(ISO2022JPEncoding.decode([0x1b][], DecoderTrap::Replace),
+                   Ok("\u{fffd}".to_string()));
+        assert_eq!(ISO2022JPEncoding.decode([0x1b, 0x24][], DecoderTrap::Replace),
+                   Ok("\u{fffd}".to_string()));
+        assert_eq!(ISO2022JPEncoding.decode([0x1b, 0x24, 0x28][], DecoderTrap::Replace),
+                   Ok("\u{fffd}\x28".to_string()));
+        assert_eq!(ISO2022JPEncoding.decode([0x1b, 0x28][], DecoderTrap::Replace),
+                   Ok("\u{fffd}".to_string()));
     }
 
     #[test]
@@ -1134,7 +1129,9 @@ mod iso2022jp_tests {
         // also tests allowed but never used escape codes in ISO 2022
         let mut d = ISO2022JPEncoding.raw_decoder();
         macro_rules! reset(() => (
-            assert_feed_ok!(d, [0x41, 0x42, 0x43, 0x1b, 0x24, 0x42, 0x21, 0x21], [], "ABC\u3000")))
+            assert_feed_ok!(d, [0x41, 0x42, 0x43, 0x1b, 0x24, 0x42, 0x21, 0x21], [],
+                            "ABC\u{3000}")
+        ))
 
         reset!()
         assert_feed_ok!(d, [], [0x1b], "");
@@ -1233,11 +1230,11 @@ mod iso2022jp_tests {
         let mut d = ISO2022JPEncoding.raw_decoder();
         assert_feed_ok!(d, [0x24, 0x22,
                             0x1b, 0x24, 0x42,
-                            0x24, 0x22], [0x24], "\x24\x22\u3042");
+                            0x24, 0x22], [0x24], "\x24\x22\u{3042}");
         assert_finish_err!(d, "");
         assert_feed_ok!(d, [0x24, 0x22,
                             0x1b, 0x24, 0x42,
-                            0x24, 0x22], [], "\x24\x22\u3042");
+                            0x24, 0x22], [], "\x24\x22\u{3042}");
         assert_finish_ok!(d, "");
     }
 
@@ -1246,16 +1243,17 @@ mod iso2022jp_tests {
         let s = testutils::JAPANESE_TEXT;
         bencher.bytes = s.len() as u64;
         bencher.iter(|| test::black_box({
-            ISO2022JPEncoding.encode(s[], EncodeStrict)
+            ISO2022JPEncoding.encode(s[], EncoderTrap::Strict)
         }))
     }
 
     #[bench]
     fn bench_decode_short_text(bencher: &mut test::Bencher) {
-        let s = ISO2022JPEncoding.encode(testutils::JAPANESE_TEXT, EncodeStrict).ok().unwrap();
+        let s = ISO2022JPEncoding.encode(testutils::JAPANESE_TEXT,
+                                         EncoderTrap::Strict).ok().unwrap();
         bencher.bytes = s.len() as u64;
         bencher.iter(|| test::black_box({
-            ISO2022JPEncoding.decode(s[], DecodeStrict)
+            ISO2022JPEncoding.decode(s[], DecoderTrap::Strict)
         }))
     }
 }
