@@ -1,5 +1,5 @@
 // This is a part of rust-encoding.
-// Copyright (c) 2013, Kang Seonghoon.
+// Copyright (c) 2013-2014, Kang Seonghoon.
 // See README.md and LICENSE.txt for details.
 //
 // Portions Copyright (c) 2008-2009 Bjoern Hoehrmann <bjoern@hoehrmann.de>
@@ -42,7 +42,7 @@ use types::*;
  * The UTF-8 scanner used by this module is heavily based on Bjoern Hoehrmann's
  * [Flexible and Economical UTF-8 Decoder](http://bjoern.hoehrmann.de/utf-8/decoder/dfa/).
  */
-#[deriving(Clone)]
+#[deriving(Clone, Copy)]
 pub struct UTF8Encoding;
 
 impl Encoding for UTF8Encoding {
@@ -53,7 +53,7 @@ impl Encoding for UTF8Encoding {
 }
 
 /// An encoder for UTF-8.
-#[deriving(Clone)]
+#[deriving(Clone, Copy)]
 pub struct UTF8Encoder;
 
 impl UTF8Encoder {
@@ -77,6 +77,7 @@ impl RawEncoder for UTF8Encoder {
 }
 
 /// A decoder for UTF-8.
+#[deriving(Clone, Copy)]
 pub struct UTF8Decoder {
     queuelen: uint,
     queue: [u8, ..4],
@@ -89,12 +90,6 @@ impl UTF8Decoder {
     }
 }
 
-impl Clone for UTF8Decoder {
-    fn clone(&self) -> UTF8Decoder {
-        UTF8Decoder { queuelen: self.queuelen, queue: self.queue, state: self.state }
-    }
-}
-
 static CHAR_CATEGORY: [u8, ..256] = [
     //  0 (00-7F): one byte sequence
     //  1 (80-8F): continuation byte
@@ -263,25 +258,25 @@ mod tests {
 
         // two bytes
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xc2, 0xa2], [], "\u00a2");
-        assert_feed_ok!(d, [0xc2, 0xac, 0xc2, 0xa9], [], "\u00ac\u00a9");
+        assert_feed_ok!(d, [0xc2, 0xa2], [], "\u{a2}");
+        assert_feed_ok!(d, [0xc2, 0xac, 0xc2, 0xa9], [], "\u{ac}\u{0a9}");
         assert_feed_ok!(d, [], [], "");
         assert_feed_ok!(d, [0xd5, 0xa1, 0xd5, 0xb5, 0xd5, 0xa2, 0xd5, 0xb8, 0xd6, 0x82,
                             0xd5, 0xa2, 0xd5, 0xa5, 0xd5, 0xb6], [],
-                        "\u0561\u0575\u0562\u0578\u0582\u0562\u0565\u0576");
+                        "\u{561}\u{0575}\u{562}\u{578}\u{582}\u{562}\u{565}\u{576}");
         assert_finish_ok!(d, "");
 
         // three bytes
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xed, 0x92, 0x89], [], "\ud489");
-        assert_feed_ok!(d, [0xe6, 0xbc, 0xa2, 0xe5, 0xad, 0x97], [], "\u6f22\u5b57");
+        assert_feed_ok!(d, [0xed, 0x92, 0x89], [], "\u{d489}");
+        assert_feed_ok!(d, [0xe6, 0xbc, 0xa2, 0xe5, 0xad, 0x97], [], "\u{6f22}\u{5b57}");
         assert_feed_ok!(d, [], [], "");
-        assert_feed_ok!(d, [0xc9, 0x99, 0xc9, 0x94, 0xc9, 0x90], [], "\u0259\u0254\u0250");
+        assert_feed_ok!(d, [0xc9, 0x99, 0xc9, 0x94, 0xc9, 0x90], [], "\u{259}\u{0254}\u{250}");
         assert_finish_ok!(d, "");
 
         // four bytes
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xf0, 0x90, 0x82, 0x82], [], "\U00010082");
+        assert_feed_ok!(d, [0xf0, 0x90, 0x82, 0x82], [], "\u{10082}");
         assert_feed_ok!(d, [], [], "");
         assert_finish_ok!(d, "");
 
@@ -299,35 +294,35 @@ mod tests {
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xc2, 0x80], [], "\u0080");
+        assert_feed_ok!(d, [0xc2, 0x80], [], "\u{80}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xdf, 0xbf], [], "\u07ff");
+        assert_feed_ok!(d, [0xdf, 0xbf], [], "\u{7ff}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xe0, 0xa0, 0x80], [], "\u0800");
+        assert_feed_ok!(d, [0xe0, 0xa0, 0x80], [], "\u{800}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xed, 0x9f, 0xbf], [], "\ud7ff");
+        assert_feed_ok!(d, [0xed, 0x9f, 0xbf], [], "\u{d7ff}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xee, 0x80, 0x80], [], "\ue000");
+        assert_feed_ok!(d, [0xee, 0x80, 0x80], [], "\u{e000}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xef, 0xbf, 0xbf], [], "\uffff");
+        assert_feed_ok!(d, [0xef, 0xbf, 0xbf], [], "\u{ffff}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xf0, 0x90, 0x80, 0x80], [], "\U00010000");
+        assert_feed_ok!(d, [0xf0, 0x90, 0x80, 0x80], [], "\u{10000}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xf4, 0x8f, 0xbf, 0xbf], [], "\U0010ffff");
+        assert_feed_ok!(d, [0xf4, 0x8f, 0xbf, 0xbf], [], "\u{10ffff}");
         assert_finish_ok!(d, "");
     }
 
@@ -337,13 +332,13 @@ mod tests {
         assert_feed_ok!(d, [], [0xf0], "");
         assert_feed_ok!(d, [], [0x90], "");
         assert_feed_ok!(d, [], [0x82], "");
-        assert_feed_ok!(d, [0x82], [0xed], "\U00010082");
-        assert_feed_ok!(d, [0x92, 0x89], [], "\ud489");
+        assert_feed_ok!(d, [0x82], [0xed], "\u{10082}");
+        assert_feed_ok!(d, [0x92, 0x89], [], "\u{d489}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
         assert_feed_ok!(d, [], [0xc2], "");
-        assert_feed_ok!(d, [0xa9, 0x20], [], "\u00a9\u0020");
+        assert_feed_ok!(d, [0xa9, 0x20], [], "\u{a9}\u{020}");
         assert_finish_ok!(d, "");
     }
 
@@ -513,15 +508,15 @@ mod tests {
     #[test]
     fn test_invalid_too_many_cont_bytes() {
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_err!(d, [0xc2, 0x80], [0x80], [], "\u0080");
+        assert_feed_err!(d, [0xc2, 0x80], [0x80], [], "\u{80}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_err!(d, [0xe0, 0xa0, 0x80], [0x80], [], "\u0800");
+        assert_feed_err!(d, [0xe0, 0xa0, 0x80], [0x80], [], "\u{800}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_err!(d, [0xf0, 0x90, 0x80, 0x80], [0x80], [], "\U00010000");
+        assert_feed_err!(d, [0xf0, 0x90, 0x80, 0x80], [0x80], [], "\u{10000}");
         assert_finish_ok!(d, "");
 
         // no continuation byte is consumed after 5/6-byte sequence starters and FE/FF
@@ -546,17 +541,17 @@ mod tests {
     fn test_invalid_too_many_cont_bytes_partial() {
         let mut d = UTF8Encoding.raw_decoder();
         assert_feed_ok!(d, [], [0xc2], "");
-        assert_feed_err!(d, [0x80], [0x80], [], "\u0080");
+        assert_feed_err!(d, [0x80], [0x80], [], "\u{80}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
         assert_feed_ok!(d, [], [0xe0, 0xa0], "");
-        assert_feed_err!(d, [0x80], [0x80], [], "\u0800");
+        assert_feed_err!(d, [0x80], [0x80], [], "\u{800}");
         assert_finish_ok!(d, "");
 
         let mut d = UTF8Encoding.raw_decoder();
         assert_feed_ok!(d, [], [0xf0, 0x90, 0x80], "");
-        assert_feed_err!(d, [0x80], [0x80], [], "\U00010000");
+        assert_feed_err!(d, [0x80], [0x80], [], "\u{10000}");
         assert_finish_ok!(d, "");
 
         // no continuation byte is consumed after 5/6-byte sequence starters and FE/FF
@@ -614,9 +609,9 @@ mod tests {
     #[test]
     fn test_feed_after_finish() {
         let mut d = UTF8Encoding.raw_decoder();
-        assert_feed_ok!(d, [0xc2, 0x80], [0xc2], "\u0080");
+        assert_feed_ok!(d, [0xc2, 0x80], [0xc2], "\u{80}");
         assert_finish_err!(d, "");
-        assert_feed_ok!(d, [0xc2, 0x80], [], "\u0080");
+        assert_feed_ok!(d, [0xc2, 0x80], [], "\u{80}");
         assert_finish_ok!(d, "");
     }
 
@@ -638,15 +633,13 @@ mod tests {
         use std::str;
         use testutils;
         use types::*;
-        use types::DecoderTrap::Strict as DecodeStrict;
-        use types::EncoderTrap::Strict as EncodeStrict;
 
         #[bench]
         fn bench_encode(bencher: &mut test::Bencher) {
             let s = testutils::ASCII_TEXT;
             bencher.bytes = s.len() as u64;
             bencher.iter(|| test::black_box({
-                UTF8Encoding.encode(s, EncodeStrict)
+                UTF8Encoding.encode(s, EncoderTrap::Strict)
             }))
         }
 
@@ -655,7 +648,7 @@ mod tests {
             let s = testutils::ASCII_TEXT.as_bytes();
             bencher.bytes = s.len() as u64;
             bencher.iter(|| test::black_box({
-                UTF8Encoding.decode(s, DecodeStrict)
+                UTF8Encoding.decode(s, DecoderTrap::Strict)
             }))
         }
 
@@ -695,15 +688,13 @@ mod tests {
         use std::str;
         use testutils;
         use types::*;
-        use types::DecoderTrap::Strict as DecodeStrict;
-        use types::EncoderTrap::Strict as EncodeStrict;
 
         #[bench]
         fn bench_encode(bencher: &mut test::Bencher) {
             let s = testutils::KOREAN_TEXT;
             bencher.bytes = s.len() as u64;
             bencher.iter(|| test::black_box({
-                UTF8Encoding.encode(s, EncodeStrict)
+                UTF8Encoding.encode(s, EncoderTrap::Strict)
             }))
         }
 
@@ -712,7 +703,7 @@ mod tests {
             let s = testutils::KOREAN_TEXT.as_bytes();
             bencher.bytes = s.len() as u64;
             bencher.iter(|| test::black_box({
-                UTF8Encoding.decode(s, DecodeStrict)
+                UTF8Encoding.decode(s, DecoderTrap::Strict)
             }))
         }
 
